@@ -55,59 +55,79 @@ Non-Parametric Function Induction in Semi-Supervised Learning. AISTAT 2005
 # Authors: Clay Woolam <clay@woolam.org>
 #          Utkarsh Upadhyay <mail@musicallyut.in>
 # License: BSD
-from abc import ABCMeta, abstractmethod
-
 import warnings
+from abc import ABCMeta, abstractmethod
+from numbers import Integral, Real
+
 import numpy as np
 from scipy import sparse
 from scipy.sparse import csgraph
 
-from ..base import BaseEstimator, ClassifierMixin
+from ..base import BaseEstimator, ClassifierMixin, _fit_context
+from ..exceptions import ConvergenceWarning
 from ..metrics.pairwise import rbf_kernel
 from ..neighbors import NearestNeighbors
+from ..utils._param_validation import Interval, StrOptions
 from ..utils.extmath import safe_sparse_dot
 from ..utils.multiclass import check_classification_targets
-from ..utils.validation import check_X_y, check_is_fitted, check_array
-from ..exceptions import ConvergenceWarning
+from ..utils.validation import check_is_fitted
 
 
 class BaseLabelPropagation(ClassifierMixin, BaseEstimator, metaclass=ABCMeta):
     """Base class for label propagation module.
 
-    Parameters
-    ----------
-    kernel : {'knn', 'rbf', callable}
-        String identifier for kernel function to use or the kernel function
-        itself. Only 'rbf' and 'knn' strings are valid inputs. The function
-        passed should take two inputs, each of shape [n_samples, n_features],
-        and return a [n_samples, n_samples] shaped weight matrix
+     Parameters
+     ----------
+     kernel : {'knn', 'rbf'} or callable, default='rbf'
+         String identifier for kernel function to use or the kernel function
+         itself. Only 'rbf' and 'knn' strings are valid inputs. The function
+         passed should take two inputs, each of shape (n_samples, n_features),
+         and return a (n_samples, n_samples) shaped weight matrix.
 
-    gamma : float
-        Parameter for rbf kernel
+     gamma : float, default=20
+         Parameter for rbf kernel.
 
-    n_neighbors : integer > 0
-        Parameter for knn kernel
+     n_neighbors : int, default=7
+         Parameter for knn kernel. Need to be strictly positive.
 
-    alpha : float
-        Clamping factor
+     alpha : float, default=1.0
+         Clamping factor.
 
-    max_iter : integer
-        Change maximum number of iterations allowed
+     max_iter : int, default=30
+         Change maximum number of iterations allowed.
 
-    tol : float
-        Convergence tolerance: threshold to consider the system at steady
-        state
+     tol : float, default=1e-3
+         Convergence tolerance: threshold to consider the system at steady
+         state.
 
-   n_jobs : int or None, optional (default=None)
-        The number of parallel jobs to run.
-        ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
-        ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
-        for more details.
+    n_jobs : int, default=None
+         The number of parallel jobs to run.
+         ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
+         ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
+         for more details.
     """
 
-    def __init__(self, kernel='rbf', gamma=20, n_neighbors=7,
-                 alpha=1, max_iter=30, tol=1e-3, n_jobs=None):
+    _parameter_constraints: dict = {
+        "kernel": [StrOptions({"knn", "rbf"}), callable],
+        "gamma": [Interval(Real, 0, None, closed="left")],
+        "n_neighbors": [Interval(Integral, 0, None, closed="neither")],
+        "alpha": [None, Interval(Real, 0, 1, closed="neither")],
+        "max_iter": [Interval(Integral, 0, None, closed="neither")],
+        "tol": [Interval(Real, 0, None, closed="left")],
+        "n_jobs": [None, Integral],
+    }
 
+    def __init__(
+        self,
+        kernel="rbf",
+        *,
+        gamma=20,
+        n_neighbors=7,
+        alpha=1,
+        max_iter=30,
+        tol=1e-3,
+        n_jobs=None,
+    ):
         self.max_iter = max_iter
         self.tol = tol
 
@@ -129,12 +149,13 @@ class BaseLabelPropagation(ClassifierMixin, BaseEstimator, metaclass=ABCMeta):
                 return rbf_kernel(X, y, gamma=self.gamma)
         elif self.kernel == "knn":
             if self.nn_fit is None:
-                self.nn_fit = NearestNeighbors(self.n_neighbors,
-                                               n_jobs=self.n_jobs).fit(X)
+                self.nn_fit = NearestNeighbors(
+                    n_neighbors=self.n_neighbors, n_jobs=self.n_jobs
+                ).fit(X)
             if y is None:
-                return self.nn_fit.kneighbors_graph(self.nn_fit._fit_X,
-                                                    self.n_neighbors,
-                                                    mode='connectivity')
+                return self.nn_fit.kneighbors_graph(
+                    self.nn_fit._fit_X, self.n_neighbors, mode="connectivity"
+                )
             else:
                 return self.nn_fit.kneighbors(y, return_distance=False)
         elif callable(self.kernel):
@@ -142,28 +163,30 @@ class BaseLabelPropagation(ClassifierMixin, BaseEstimator, metaclass=ABCMeta):
                 return self.kernel(X, X)
             else:
                 return self.kernel(X, y)
-        else:
-            raise ValueError("%s is not a valid kernel. Only rbf and knn"
-                             " or an explicit function "
-                             " are supported at this time." % self.kernel)
 
     @abstractmethod
     def _build_graph(self):
-        raise NotImplementedError("Graph construction must be implemented"
-                                  " to fit a label propagation model.")
+        raise NotImplementedError(
+            "Graph construction must be implemented to fit a label propagation model."
+        )
 
     def predict(self, X):
-        """Performs inductive inference across the model.
+        """Perform inductive inference across the model.
 
         Parameters
         ----------
         X : array-like of shape (n_samples, n_features)
+            The data matrix.
 
         Returns
         -------
-        y : array_like, shape = [n_samples]
-            Predictions for input data
+        y : ndarray of shape (n_samples,)
+            Predictions for input data.
         """
+        # Note: since `predict` does not accept semi-supervised labels as input,
+        # `fit(X, y).predict(X) != fit(X, y).transduction_`.
+        # Hence, `fit_predict` is not implemented.
+        # See https://github.com/scikit-learn/scikit-learn/pull/24898
         probas = self.predict_proba(X)
         return self.classes_[np.argmax(probas, axis=1)].ravel()
 
@@ -177,50 +200,67 @@ class BaseLabelPropagation(ClassifierMixin, BaseEstimator, metaclass=ABCMeta):
         Parameters
         ----------
         X : array-like of shape (n_samples, n_features)
+            The data matrix.
 
         Returns
         -------
-        probabilities : array, shape = [n_samples, n_classes]
+        probabilities : ndarray of shape (n_samples, n_classes)
             Normalized probability distributions across
-            class labels
+            class labels.
         """
         check_is_fitted(self)
 
-        X_2d = check_array(X, accept_sparse=['csc', 'csr', 'coo', 'dok',
-                                             'bsr', 'lil', 'dia'])
+        X_2d = self._validate_data(
+            X,
+            accept_sparse=["csc", "csr", "coo", "dok", "bsr", "lil", "dia"],
+            reset=False,
+        )
         weight_matrices = self._get_kernel(self.X_, X_2d)
-        if self.kernel == 'knn':
-            probabilities = np.array([
-                np.sum(self.label_distributions_[weight_matrix], axis=0)
-                for weight_matrix in weight_matrices])
+        if self.kernel == "knn":
+            probabilities = np.array(
+                [
+                    np.sum(self.label_distributions_[weight_matrix], axis=0)
+                    for weight_matrix in weight_matrices
+                ]
+            )
         else:
             weight_matrices = weight_matrices.T
-            probabilities = np.dot(weight_matrices, self.label_distributions_)
+            probabilities = safe_sparse_dot(weight_matrices, self.label_distributions_)
         normalizer = np.atleast_2d(np.sum(probabilities, axis=1)).T
         probabilities /= normalizer
         return probabilities
 
+    @_fit_context(prefer_skip_nested_validation=True)
     def fit(self, X, y):
-        """Fit a semi-supervised label propagation model based
+        """Fit a semi-supervised label propagation model to X.
 
-        All the input data is provided matrix X (labeled and unlabeled)
-        and corresponding label matrix y with a dedicated marker value for
-        unlabeled samples.
+        The input samples (labeled and unlabeled) are provided by matrix X,
+        and target labels are provided by matrix y. We conventionally apply the
+        label -1 to unlabeled samples in matrix y in a semi-supervised
+        classification.
 
         Parameters
         ----------
-        X : array-like of shape (n_samples, n_features)
-            A {n_samples by n_samples} size matrix will be created from this
+        X : {array-like, sparse matrix} of shape (n_samples, n_features)
+            Training data, where `n_samples` is the number of samples
+            and `n_features` is the number of features.
 
-        y : array_like, shape = [n_samples]
-            n_labeled_samples (unlabeled points are marked as -1)
+        y : array-like of shape (n_samples,)
+            Target class values with unlabeled points marked as -1.
             All unlabeled samples will be transductively assigned labels
+            internally, which are stored in `transduction_`.
 
         Returns
         -------
-        self : returns an instance of self.
+        self : object
+            Returns the instance itself.
         """
-        X, y = check_X_y(X, y)
+        X, y = self._validate_data(
+            X,
+            y,
+            accept_sparse=["csr", "csc"],
+            reset=True,
+        )
         self.X_ = X
         check_classification_targets(y)
 
@@ -230,16 +270,11 @@ class BaseLabelPropagation(ClassifierMixin, BaseEstimator, metaclass=ABCMeta):
         # label construction
         # construct a categorical distribution for classification only
         classes = np.unique(y)
-        classes = (classes[classes != -1])
+        classes = classes[classes != -1]
         self.classes_ = classes
 
         n_samples, n_classes = len(y), len(classes)
 
-        alpha = self.alpha
-        if self._variant == 'spreading' and \
-                (alpha is None or alpha <= 0.0 or alpha >= 1.0):
-            raise ValueError('alpha=%s is invalid: it must be inside '
-                             'the open interval (0, 1)' % alpha)
         y = np.asarray(y)
         unlabeled = y == -1
 
@@ -249,12 +284,12 @@ class BaseLabelPropagation(ClassifierMixin, BaseEstimator, metaclass=ABCMeta):
             self.label_distributions_[y == label, classes == label] = 1
 
         y_static = np.copy(self.label_distributions_)
-        if self._variant == 'propagation':
+        if self._variant == "propagation":
             # LabelPropagation
             y_static[unlabeled] = 0
         else:
             # LabelSpreading
-            y_static *= 1 - alpha
+            y_static *= 1 - self.alpha
 
         l_previous = np.zeros((self.X_.shape[0], n_classes))
 
@@ -268,63 +303,65 @@ class BaseLabelPropagation(ClassifierMixin, BaseEstimator, metaclass=ABCMeta):
 
             l_previous = self.label_distributions_
             self.label_distributions_ = safe_sparse_dot(
-                graph_matrix, self.label_distributions_)
+                graph_matrix, self.label_distributions_
+            )
 
-            if self._variant == 'propagation':
-                normalizer = np.sum(
-                    self.label_distributions_, axis=1)[:, np.newaxis]
+            if self._variant == "propagation":
+                normalizer = np.sum(self.label_distributions_, axis=1)[:, np.newaxis]
+                normalizer[normalizer == 0] = 1
                 self.label_distributions_ /= normalizer
-                self.label_distributions_ = np.where(unlabeled,
-                                                     self.label_distributions_,
-                                                     y_static)
+                self.label_distributions_ = np.where(
+                    unlabeled, self.label_distributions_, y_static
+                )
             else:
                 # clamp
-                self.label_distributions_ = np.multiply(
-                    alpha, self.label_distributions_) + y_static
+                self.label_distributions_ = (
+                    np.multiply(self.alpha, self.label_distributions_) + y_static
+                )
         else:
             warnings.warn(
-                'max_iter=%d was reached without convergence.' % self.max_iter,
-                category=ConvergenceWarning
+                "max_iter=%d was reached without convergence." % self.max_iter,
+                category=ConvergenceWarning,
             )
             self.n_iter_ += 1
 
         normalizer = np.sum(self.label_distributions_, axis=1)[:, np.newaxis]
+        normalizer[normalizer == 0] = 1
         self.label_distributions_ /= normalizer
 
         # set the transduction item
-        transduction = self.classes_[np.argmax(self.label_distributions_,
-                                               axis=1)]
+        transduction = self.classes_[np.argmax(self.label_distributions_, axis=1)]
         self.transduction_ = transduction.ravel()
         return self
 
 
 class LabelPropagation(BaseLabelPropagation):
-    """Label Propagation classifier
+    """Label Propagation classifier.
 
     Read more in the :ref:`User Guide <label_propagation>`.
 
     Parameters
     ----------
-    kernel : {'knn', 'rbf', callable}
+    kernel : {'knn', 'rbf'} or callable, default='rbf'
         String identifier for kernel function to use or the kernel function
         itself. Only 'rbf' and 'knn' strings are valid inputs. The function
-        passed should take two inputs, each of shape [n_samples, n_features],
-        and return a [n_samples, n_samples] shaped weight matrix.
+        passed should take two inputs, each of shape (n_samples, n_features),
+        and return a (n_samples, n_samples) shaped weight matrix.
 
-    gamma : float
-        Parameter for rbf kernel
+    gamma : float, default=20
+        Parameter for rbf kernel.
 
-    n_neighbors : integer > 0
-        Parameter for knn kernel
+    n_neighbors : int, default=7
+        Parameter for knn kernel which need to be strictly positive.
 
-    max_iter : integer
-        Change maximum number of iterations allowed
+    max_iter : int, default=1000
+        Change maximum number of iterations allowed.
 
-    tol : float
+    tol : float, 1e-3
         Convergence tolerance: threshold to consider the system at steady
-        state
+        state.
 
-    n_jobs : int or None, optional (default=None)
+    n_jobs : int, default=None
         The number of parallel jobs to run.
         ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
         ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
@@ -332,21 +369,43 @@ class LabelPropagation(BaseLabelPropagation):
 
     Attributes
     ----------
-    X_ : array, shape = [n_samples, n_features]
+    X_ : {array-like, sparse matrix} of shape (n_samples, n_features)
         Input array.
 
-    classes_ : array, shape = [n_classes]
+    classes_ : ndarray of shape (n_classes,)
         The distinct labels used in classifying instances.
 
-    label_distributions_ : array, shape = [n_samples, n_classes]
+    label_distributions_ : ndarray of shape (n_samples, n_classes)
         Categorical distribution for each item.
 
-    transduction_ : array, shape = [n_samples]
-        Label assigned to each item via the transduction.
+    transduction_ : ndarray of shape (n_samples)
+        Label assigned to each item during :term:`fit`.
+
+    n_features_in_ : int
+        Number of features seen during :term:`fit`.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Defined only when `X`
+        has feature names that are all strings.
+
+        .. versionadded:: 1.0
 
     n_iter_ : int
         Number of iterations run.
 
+    See Also
+    --------
+    BaseLabelPropagation : Base class for label propagation module.
+    LabelSpreading : Alternate label propagation strategy more robust to noise.
+
+    References
+    ----------
+    Xiaojin Zhu and Zoubin Ghahramani. Learning from labeled and unlabeled data
+    with label propagation. Technical Report CMU-CALD-02-107, Carnegie Mellon
+    University, 2002 http://pages.cs.wisc.edu/~jerryzhu/pub/CMU-CALD-02-107.pdf
+
     Examples
     --------
     >>> import numpy as np
@@ -360,25 +419,32 @@ class LabelPropagation(BaseLabelPropagation):
     >>> labels[random_unlabeled_points] = -1
     >>> label_prop_model.fit(iris.data, labels)
     LabelPropagation(...)
-
-    References
-    ----------
-    Xiaojin Zhu and Zoubin Ghahramani. Learning from labeled and unlabeled data
-    with label propagation. Technical Report CMU-CALD-02-107, Carnegie Mellon
-    University, 2002 http://pages.cs.wisc.edu/~jerryzhu/pub/CMU-CALD-02-107.pdf
-
-    See Also
-    --------
-    LabelSpreading : Alternate label propagation strategy more robust to noise
     """
 
-    _variant = 'propagation'
+    _variant = "propagation"
 
-    def __init__(self, kernel='rbf', gamma=20, n_neighbors=7,
-                 max_iter=1000, tol=1e-3, n_jobs=None):
-        super().__init__(kernel=kernel, gamma=gamma,
-                         n_neighbors=n_neighbors, max_iter=max_iter,
-                         tol=tol, n_jobs=n_jobs, alpha=None)
+    _parameter_constraints: dict = {**BaseLabelPropagation._parameter_constraints}
+    _parameter_constraints.pop("alpha")
+
+    def __init__(
+        self,
+        kernel="rbf",
+        *,
+        gamma=20,
+        n_neighbors=7,
+        max_iter=1000,
+        tol=1e-3,
+        n_jobs=None,
+    ):
+        super().__init__(
+            kernel=kernel,
+            gamma=gamma,
+            n_neighbors=n_neighbors,
+            max_iter=max_iter,
+            tol=tol,
+            n_jobs=n_jobs,
+            alpha=None,
+        )
 
     def _build_graph(self):
         """Matrix representing a fully connected graph between each sample
@@ -386,7 +452,7 @@ class LabelPropagation(BaseLabelPropagation):
         This basic implementation creates a non-stochastic affinity matrix, so
         class distributions will exceed 1 (normalization may be desired).
         """
-        if self.kernel == 'knn':
+        if self.kernel == "knn":
             self.nn_fit = None
         affinity_matrix = self._get_kernel(self.X_)
         normalizer = affinity_matrix.sum(axis=0)
@@ -397,11 +463,29 @@ class LabelPropagation(BaseLabelPropagation):
         return affinity_matrix
 
     def fit(self, X, y):
+        """Fit a semi-supervised label propagation model to X.
+
+        Parameters
+        ----------
+        X : {array-like, sparse matrix} of shape (n_samples, n_features)
+            Training data, where `n_samples` is the number of samples
+            and `n_features` is the number of features.
+
+        y : array-like of shape (n_samples,)
+            Target class values with unlabeled points marked as -1.
+            All unlabeled samples will be transductively assigned labels
+            internally, which are stored in `transduction_`.
+
+        Returns
+        -------
+        self : object
+            Returns the instance itself.
+        """
         return super().fit(X, y)
 
 
 class LabelSpreading(BaseLabelPropagation):
-    """LabelSpreading model for semi-supervised learning
+    """LabelSpreading model for semi-supervised learning.
 
     This model is similar to the basic Label Propagation algorithm,
     but uses affinity matrix based on the normalized graph Laplacian
@@ -411,33 +495,33 @@ class LabelSpreading(BaseLabelPropagation):
 
     Parameters
     ----------
-    kernel : {'knn', 'rbf', callable}
+    kernel : {'knn', 'rbf'} or callable, default='rbf'
         String identifier for kernel function to use or the kernel function
         itself. Only 'rbf' and 'knn' strings are valid inputs. The function
-        passed should take two inputs, each of shape [n_samples, n_features],
-        and return a [n_samples, n_samples] shaped weight matrix
+        passed should take two inputs, each of shape (n_samples, n_features),
+        and return a (n_samples, n_samples) shaped weight matrix.
 
-    gamma : float
-      parameter for rbf kernel
+    gamma : float, default=20
+      Parameter for rbf kernel.
 
-    n_neighbors : integer > 0
-      parameter for knn kernel
+    n_neighbors : int, default=7
+      Parameter for knn kernel which is a strictly positive integer.
 
-    alpha : float
+    alpha : float, default=0.2
       Clamping factor. A value in (0, 1) that specifies the relative amount
       that an instance should adopt the information from its neighbors as
       opposed to its initial label.
       alpha=0 means keeping the initial label information; alpha=1 means
       replacing all initial information.
 
-    max_iter : integer
-      maximum number of iterations allowed
+    max_iter : int, default=30
+      Maximum number of iterations allowed.
 
-    tol : float
+    tol : float, default=1e-3
       Convergence tolerance: threshold to consider the system at steady
-      state
+      state.
 
-    n_jobs : int or None, optional (default=None)
+    n_jobs : int, default=None
         The number of parallel jobs to run.
         ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
         ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
@@ -445,21 +529,42 @@ class LabelSpreading(BaseLabelPropagation):
 
     Attributes
     ----------
-    X_ : array, shape = [n_samples, n_features]
+    X_ : ndarray of shape (n_samples, n_features)
         Input array.
 
-    classes_ : array, shape = [n_classes]
+    classes_ : ndarray of shape (n_classes,)
         The distinct labels used in classifying instances.
 
-    label_distributions_ : array, shape = [n_samples, n_classes]
+    label_distributions_ : ndarray of shape (n_samples, n_classes)
         Categorical distribution for each item.
 
-    transduction_ : array, shape = [n_samples]
-        Label assigned to each item via the transduction.
+    transduction_ : ndarray of shape (n_samples,)
+        Label assigned to each item during :term:`fit`.
+
+    n_features_in_ : int
+        Number of features seen during :term:`fit`.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Defined only when `X`
+        has feature names that are all strings.
+
+        .. versionadded:: 1.0
 
     n_iter_ : int
         Number of iterations run.
 
+    See Also
+    --------
+    LabelPropagation : Unregularized graph based semi-supervised learning.
+
+    References
+    ----------
+    `Dengyong Zhou, Olivier Bousquet, Thomas Navin Lal, Jason Weston,
+    Bernhard Schoelkopf. Learning with local and global consistency (2004)
+    <https://citeseerx.ist.psu.edu/doc_view/pid/d74c37aabf2d5cae663007cbd8718175466aea8c>`_
+
     Examples
     --------
     >>> import numpy as np
@@ -473,40 +578,47 @@ class LabelSpreading(BaseLabelPropagation):
     >>> labels[random_unlabeled_points] = -1
     >>> label_prop_model.fit(iris.data, labels)
     LabelSpreading(...)
-
-    References
-    ----------
-    Dengyong Zhou, Olivier Bousquet, Thomas Navin Lal, Jason Weston,
-    Bernhard Schoelkopf. Learning with local and global consistency (2004)
-    http://citeseer.ist.psu.edu/viewdoc/summary?doi=10.1.1.115.3219
-
-    See Also
-    --------
-    LabelPropagation : Unregularized graph based semi-supervised learning
     """
 
-    _variant = 'spreading'
+    _variant = "spreading"
 
-    def __init__(self, kernel='rbf', gamma=20, n_neighbors=7, alpha=0.2,
-                 max_iter=30, tol=1e-3, n_jobs=None):
+    _parameter_constraints: dict = {**BaseLabelPropagation._parameter_constraints}
+    _parameter_constraints["alpha"] = [Interval(Real, 0, 1, closed="neither")]
 
+    def __init__(
+        self,
+        kernel="rbf",
+        *,
+        gamma=20,
+        n_neighbors=7,
+        alpha=0.2,
+        max_iter=30,
+        tol=1e-3,
+        n_jobs=None,
+    ):
         # this one has different base parameters
-        super().__init__(kernel=kernel, gamma=gamma,
-                         n_neighbors=n_neighbors, alpha=alpha,
-                         max_iter=max_iter, tol=tol, n_jobs=n_jobs)
+        super().__init__(
+            kernel=kernel,
+            gamma=gamma,
+            n_neighbors=n_neighbors,
+            alpha=alpha,
+            max_iter=max_iter,
+            tol=tol,
+            n_jobs=n_jobs,
+        )
 
     def _build_graph(self):
         """Graph matrix for Label Spreading computes the graph laplacian"""
         # compute affinity matrix (or gram matrix)
-        if self.kernel == 'knn':
+        if self.kernel == "knn":
             self.nn_fit = None
         n_samples = self.X_.shape[0]
         affinity_matrix = self._get_kernel(self.X_)
         laplacian = csgraph.laplacian(affinity_matrix, normed=True)
         laplacian = -laplacian
         if sparse.isspmatrix(laplacian):
-            diag_mask = (laplacian.row == laplacian.col)
+            diag_mask = laplacian.row == laplacian.col
             laplacian.data[diag_mask] = 0.0
         else:
-            laplacian.flat[::n_samples + 1] = 0.0  # set diag to 0.0
+            laplacian.flat[:: n_samples + 1] = 0.0  # set diag to 0.0
         return laplacian
