@@ -2,31 +2,38 @@
 #          Raghav RV <rvraghav93@gmail.com>
 # License: BSD 3 clause
 
+import importlib
 import inspect
 import warnings
-import importlib
-
-from pkgutil import walk_packages
 from inspect import signature
+from pkgutil import walk_packages
 
 import numpy as np
-
-import sklearn
-from sklearn.utils import IS_PYPY
-from sklearn.utils._testing import check_docstring_parameters
-from sklearn.utils._testing import _get_func_name
-from sklearn.utils._testing import ignore_warnings
-from sklearn.utils import all_estimators
-from sklearn.utils.estimator_checks import _enforce_estimator_tags_y
-from sklearn.utils.estimator_checks import _enforce_estimator_tags_x
-from sklearn.utils.estimator_checks import _construct_instance
-from sklearn.utils.deprecation import _is_deprecated
-from sklearn.datasets import make_classification
-from sklearn.linear_model import LogisticRegression
-from sklearn.preprocessing import FunctionTransformer
-
 import pytest
 
+import sklearn
+from sklearn.datasets import make_classification
+
+# make it possible to discover experimental estimators when calling `all_estimators`
+from sklearn.experimental import (
+    enable_halving_search_cv,  # noqa
+    enable_iterative_imputer,  # noqa
+)
+from sklearn.linear_model import LogisticRegression
+from sklearn.preprocessing import FunctionTransformer
+from sklearn.utils import IS_PYPY, all_estimators
+from sklearn.utils._testing import (
+    _get_func_name,
+    check_docstring_parameters,
+    ignore_warnings,
+)
+from sklearn.utils.deprecation import _is_deprecated
+from sklearn.utils.estimator_checks import (
+    _construct_instance,
+    _enforce_estimator_tags_X,
+    _enforce_estimator_tags_y,
+)
+from sklearn.utils.fixes import parse_version, sp_version
 
 # walk_packages() ignores DeprecationWarnings, now we need to ignore
 # FutureWarnings
@@ -104,12 +111,11 @@ def test_docstring_parameters():
                     "Error for __init__ of %s in %s:\n%s" % (cls, name, w[0])
                 )
 
-            cls_init = getattr(cls, "__init__", None)
-
-            if _is_deprecated(cls_init):
+            # Skip checks on deprecated classes
+            if _is_deprecated(cls.__new__):
                 continue
-            elif cls_init is not None:
-                this_incorrect += check_docstring_parameters(cls.__init__, cdoc)
+
+            this_incorrect += check_docstring_parameters(cls.__init__, cdoc)
 
             for method_name in cdoc.methods:
                 method = getattr(cls, method_name)
@@ -151,7 +157,6 @@ def test_docstring_parameters():
 def test_tabs():
     # Test that there are no tabs in our source files
     for importer, modname, ispkg in walk_packages(sklearn.__path__, prefix="sklearn."):
-
         if IS_PYPY and (
             "_svmlight_format_io" in modname
             or "feature_extraction._hashing_fast" in modname
@@ -194,6 +199,7 @@ def _construct_sparse_coder(Estimator):
     return Estimator(dictionary=dictionary)
 
 
+@ignore_warnings(category=sklearn.exceptions.ConvergenceWarning)
 @pytest.mark.parametrize("name, Estimator", all_estimators())
 def test_fit_docstring_attributes(name, Estimator):
     pytest.importorskip("numpydoc")
@@ -233,34 +239,42 @@ def test_fit_docstring_attributes(name, Estimator):
     ):
         # default="auto" raises an error with the shape of `X`
         est.set_params(n_components=2)
+    elif Estimator.__name__ == "TSNE":
+        # default raises an error, perplexity must be less than n_samples
+        est.set_params(perplexity=2)
 
-    # FIXME: TO BE REMOVED in 1.4 (avoid FutureWarning)
+    # TODO(1.4): TO BE REMOVED for 1.4 (avoid FutureWarning)
+    if Estimator.__name__ in ("KMeans", "MiniBatchKMeans"):
+        est.set_params(n_init="auto")
+
+    # TODO(1.4): TO BE REMOVED for 1.5 (avoid FutureWarning)
+    if Estimator.__name__ in ("LinearSVC", "LinearSVR"):
+        est.set_params(dual="auto")
+
+    # TODO(1.4): TO BE REMOVED for 1.4 (avoid FutureWarning)
     if Estimator.__name__ in (
-        "OrthogonalMatchingPursuit",
-        "OrthogonalMatchingPursuitCV",
-        "Lars",
-        "LarsCV",
-        "LassoLars",
-        "LassoLarsCV",
-        "LassoLarsIC",
+        "MultinomialNB",
+        "ComplementNB",
+        "BernoulliNB",
+        "CategoricalNB",
     ):
-        est.set_params(normalize=False)
+        est.set_params(force_alpha=True)
 
-    # FIXME: TO BE REMOVED for 1.2 (avoid FutureWarning)
-    if Estimator.__name__ == "TSNE":
-        est.set_params(learning_rate=200.0, init="random")
+    if Estimator.__name__ == "QuantileRegressor":
+        solver = "highs" if sp_version >= parse_version("1.6.0") else "interior-point"
+        est.set_params(solver=solver)
 
-    # FIXME: TO BE REMOVED for 1.3 (avoid FutureWarning)
-    if Estimator.__name__ == "SequentialFeatureSelector":
-        est.set_params(n_features_to_select="auto")
+    # TODO(1.4): TO BE REMOVED for 1.4 (avoid FutureWarning)
+    if Estimator.__name__ == "MDS":
+        est.set_params(normalized_stress="auto")
 
-    # FIXME: TO BE REMOVED for 1.3 (avoid FutureWarning)
-    if Estimator.__name__ == "FastICA":
-        est.set_params(whiten="unit-variance")
+    # Low max iter to speed up tests: we are only interested in checking the existence
+    # of fitted attributes. This should be invariant to whether it has converged or not.
+    if "max_iter" in est.get_params():
+        est.set_params(max_iter=2)
 
-    # FIXME: TO BE REMOVED for 1.3 (avoid FutureWarning)
-    if Estimator.__name__ == "MiniBatchDictionaryLearning":
-        est.set_params(batch_size=5)
+    if "random_state" in est.get_params():
+        est.set_params(random_state=0)
 
     # In case we want to deprecate some attributes in the future
     skipped_attributes = {}
@@ -291,12 +305,14 @@ def test_fit_docstring_attributes(name, Estimator):
         )
 
         y = _enforce_estimator_tags_y(est, y)
-        X = _enforce_estimator_tags_x(est, X)
+        X = _enforce_estimator_tags_X(est, X)
 
     if "1dlabels" in est._get_tags()["X_types"]:
         est.fit(y)
     elif "2dlabels" in est._get_tags()["X_types"]:
         est.fit(np.c_[y, y])
+    elif "3darray" in est._get_tags()["X_types"]:
+        est.fit(X[np.newaxis, ...], y)
     else:
         est.fit(X, y)
 
