@@ -6,23 +6,20 @@ Multi-class / multi-label utility function
 ==========================================
 
 """
+import warnings
 from collections.abc import Sequence
 from itertools import chain
-import warnings
-
-from scipy.sparse import issparse
-from scipy.sparse import dok_matrix
-from scipy.sparse import lil_matrix
 
 import numpy as np
+from scipy.sparse import issparse, isspmatrix_dok, isspmatrix_lil
 
-from .validation import check_array, _assert_all_finite
 from ..utils._array_api import get_namespace
+from .validation import _assert_all_finite, check_array
 
 
 def _unique_multiclass(y):
-    xp, is_array_api = get_namespace(y)
-    if hasattr(y, "__array__") or is_array_api:
+    xp, is_array_api_compliant = get_namespace(y)
+    if hasattr(y, "__array__") or is_array_api_compliant:
         return xp.unique_values(xp.asarray(y))
     else:
         return set(y)
@@ -73,7 +70,7 @@ def unique_labels(*ys):
     >>> unique_labels([1, 2, 10], [5, 11])
     array([ 1,  2,  5, 10, 11])
     """
-    xp, is_array_api = get_namespace(*ys)
+    xp, is_array_api_compliant = get_namespace(*ys)
     if not ys:
         raise ValueError("No argument has been passed.")
     # Check that we don't mix label format
@@ -106,7 +103,7 @@ def unique_labels(*ys):
     if not _unique_labels:
         raise ValueError("Unknown label type: %s" % repr(ys))
 
-    if is_array_api:
+    if is_array_api_compliant:
         # array_api does not allow for mixed dtypes
         unique_ys = xp.concat([_unique_labels(y) for y in ys])
         return xp.unique_values(unique_ys)
@@ -151,8 +148,8 @@ def is_multilabel(y):
     >>> is_multilabel(np.array([[1, 0, 0]]))
     True
     """
-    xp, is_array_api = get_namespace(y)
-    if hasattr(y, "__array__") or isinstance(y, Sequence) or is_array_api:
+    xp, is_array_api_compliant = get_namespace(y)
+    if hasattr(y, "__array__") or isinstance(y, Sequence) or is_array_api_compliant:
         # DeprecationWarning will be replaced by ValueError, see NEP 34
         # https://numpy.org/neps/nep-0034-infer-dtype-is-object.html
         check_y_kwargs = dict(
@@ -179,7 +176,7 @@ def is_multilabel(y):
         return False
 
     if issparse(y):
-        if isinstance(y, (dok_matrix, lil_matrix)):
+        if isspmatrix_dok(y) or isspmatrix_lil(y):
             y = y.tocsr()
         labels = xp.unique_values(y.data)
         return (
@@ -215,7 +212,11 @@ def check_classification_targets(y):
         "multilabel-indicator",
         "multilabel-sequences",
     ]:
-        raise ValueError("Unknown label type: %r" % y_type)
+        raise ValueError(
+            f"Unknown label type: {y_type}. Maybe you are trying to fit a "
+            "classifier, which expects discrete classes on a "
+            "regression target with continuous values."
+        )
 
 
 def type_of_target(y, input_name=""):
@@ -290,11 +291,11 @@ def type_of_target(y, input_name=""):
     >>> type_of_target(np.array([[0, 1], [1, 1]]))
     'multilabel-indicator'
     """
-    xp, is_array_api = get_namespace(y)
+    xp, is_array_api_compliant = get_namespace(y)
     valid = (
         (isinstance(y, Sequence) or issparse(y) or hasattr(y, "__array__"))
         and not isinstance(y, str)
-        or is_array_api
+        or is_array_api_compliant
     )
 
     if not valid:
@@ -374,10 +375,10 @@ def type_of_target(y, input_name=""):
         suffix = ""  # [1, 2, 3] or [[1], [2], [3]]
 
     # Check float and contains non-integer float values
-    if y.dtype.kind == "f":
+    if xp.isdtype(y.dtype, "real floating"):
         # [.1, .2, 3] or [[.1, .2, 3]] or [[1., .2]] and not [1., 2., 3.]
         data = y.data if issparse(y) else y
-        if xp.any(data != data.astype(int)):
+        if xp.any(data != xp.astype(data, int)):
             _assert_all_finite(data, input_name=input_name)
             return "continuous" + suffix
 
